@@ -135,16 +135,13 @@ async fn open_connection() -> ei::Context {
         remote_desktop
             .select_devices(
                 &session,
-                (DeviceType::Keyboard | DeviceType::Pointer | DeviceType::Touchscreen).into(),
+                DeviceType::Keyboard | DeviceType::Pointer | DeviceType::Touchscreen,
                 None,
                 ashpd::desktop::PersistMode::DoNot,
             )
             .await
             .unwrap();
-        remote_desktop
-            .start(&session, &ashpd::WindowIdentifier::default())
-            .await
-            .unwrap();
+        remote_desktop.start(&session, None).await.unwrap();
         let fd = remote_desktop.connect_to_eis(&session).await.unwrap();
         let stream = UnixStream::from(fd);
         ei::Context::new(stream).unwrap()
