@@ -1,10 +1,7 @@
 use ashpd::desktop::input_capture::{Barrier, Capabilities, InputCapture};
 use pollster::FutureExt as _;
-use reis::{
-    ei,
-    event::{DeviceCapability, EiConvertEventIterator},
-};
-use std::os::unix::net::UnixStream;
+use reis::{ei, event::DeviceCapability};
+use std::{num::NonZero, os::unix::net::UnixStream};
 
 async fn open_connection() -> ei::Context {
     if let Some(context) = ei::Context::connect_to_env().unwrap() {
@@ -14,8 +11,8 @@ async fn open_connection() -> ei::Context {
         let input_capture = InputCapture::new().await.unwrap();
         let session = input_capture
             .create_session(
-                &ashpd::WindowIdentifier::default(),
-                (Capabilities::Keyboard | Capabilities::Pointer | Capabilities::Touchscreen).into(),
+                None,
+                Capabilities::Keyboard | Capabilities::Pointer | Capabilities::Touchscreen,
             )
             .await
             .unwrap()
@@ -38,7 +35,7 @@ async fn open_connection() -> ei::Context {
                 let y = region.y_offset();
                 let w = region.width() as i32;
                 let h = region.height() as i32;
-                Barrier::new(n as u32 + 1, (x, y, x + w - 1, y))
+                Barrier::new(NonZero::new(n as u32 + 1).unwrap(), (x, y, x + w - 1, y))
             })
             .collect::<Vec<_>>();
         let resp = input_capture
@@ -57,11 +54,11 @@ async fn open_connection() -> ei::Context {
 
 fn main() {
     let context = open_connection().block_on();
-    let (_connection, mut events) = context
+    let (_connection, events) = context
         .handshake_blocking("receive-example", ei::handshake::ContextType::Receiver)
         .unwrap();
 
-    while let Some(event) = events.next() {
+    for event in events {
         let event = event.unwrap();
         println!("{event:?}");
         match &event {
