@@ -118,6 +118,13 @@ impl Connection {
     }
 }
 
+// TODO libei has a `eis_clock_set_now_func`
+// Return time in us
+fn eis_now() -> u64 {
+    let time = rustix::time::clock_gettime(rustix::time::ClockId::Monotonic);
+    time.tv_sec as u64 * 1_000_000 + time.tv_nsec as u64 / 1_000
+}
+
 // need way to add seat/device?
 #[derive(Debug)]
 pub struct EisRequestConverter {
@@ -150,6 +157,14 @@ impl EisRequestConverter {
         &self.handle
     }
 
+    fn queue_frame_event(&mut self, device: &Device) {
+        self.queue_request(EisRequest::Frame(Frame {
+            time: eis_now(),
+            device: device.clone(),
+            last_serial: self.handle.last_serial(),
+        }))
+    }
+
     // Based on behavior of `eis_queue_request` in libeis
     fn queue_request(&mut self, mut request: EisRequest) {
         if request.time_mut().is_some() {
@@ -164,7 +179,11 @@ impl EisRequestConverter {
             }
             self.requests.push_back(request);
         } else {
-            // TODO: If a device request, queue a frame if anything is pending
+            if let Some(device) = request.device() {
+                if !self.pending_requests.is_empty() {
+                    self.queue_frame_event(device);
+                }
+            }
             self.requests.push_back(request);
         }
     }
@@ -389,6 +408,19 @@ impl EisRequestConverter {
                             time: 0,
                         }));
                     }
+                    eis::touchscreen::Request::Cancel { touchid } => {
+                        if touchscreen.version() < 2 {
+                            return Err(Error::InvalidInterfaceVersion(
+                                "ei_touchscreen",
+                                touchscreen.version(),
+                            ));
+                        }
+                        self.queue_request(EisRequest::TouchCancel(TouchCancel {
+                            device,
+                            touch_id: touchid,
+                            time: 0,
+                        }));
+                    }
                 }
             }
         }
@@ -406,6 +438,18 @@ struct SeatInner {
 #[derive(Clone)]
 pub struct Seat(Arc<SeatInner>);
 
+fn add_interface<I: eis::Interface>(
+    device: &eis::Device,
+    connection: Option<&Connection>,
+) -> Object {
+    // TODO better way to handle dead connection?
+    let version = connection
+        .as_ref()
+        .and_then(|c| c.interface_version(I::NAME))
+        .unwrap_or(1);
+    device.interface::<I>(version).as_object().clone()
+}
+
 impl Seat {
     pub fn eis_seat(&self) -> &eis::Seat {
         &self.0.seat
@@ -420,6 +464,8 @@ impl Seat {
         // TODO: better solution; keymap, etc.
         before_done_cb: impl for<'a> FnOnce(&'a Device),
     ) -> Device {
+        let connection = self.0.handle.upgrade().map(Connection);
+
         let device = self.0.seat.device(1);
         if let Some(name) = name {
             device.name(name);
@@ -433,12 +479,24 @@ impl Seat {
         let mut interfaces = HashMap::new();
         for capability in capabilities {
             let object = match capability {
-                DeviceCapability::Pointer => device.interface::<eis::Pointer>(1).0,
-                DeviceCapability::PointerAbsolute => device.interface::<eis::PointerAbsolute>(1).0,
-                DeviceCapability::Keyboard => device.interface::<eis::Keyboard>(1).0,
-                DeviceCapability::Touch => device.interface::<eis::Touchscreen>(1).0,
-                DeviceCapability::Scroll => device.interface::<eis::Scroll>(1).0,
-                DeviceCapability::Button => device.interface::<eis::Button>(1).0,
+                DeviceCapability::Pointer => {
+                    add_interface::<eis::Pointer>(&device, connection.as_ref())
+                }
+                DeviceCapability::PointerAbsolute => {
+                    add_interface::<eis::PointerAbsolute>(&device, connection.as_ref())
+                }
+                DeviceCapability::Keyboard => {
+                    add_interface::<eis::Keyboard>(&device, connection.as_ref())
+                }
+                DeviceCapability::Touch => {
+                    add_interface::<eis::Touchscreen>(&device, connection.as_ref())
+                }
+                DeviceCapability::Scroll => {
+                    add_interface::<eis::Scroll>(&device, connection.as_ref())
+                }
+                DeviceCapability::Button => {
+                    add_interface::<eis::Button>(&device, connection.as_ref())
+                }
             };
             interfaces.insert(object.interface().to_string(), object);
         }
@@ -450,7 +508,7 @@ impl Seat {
             interfaces,
             handle: self.0.handle.clone(),
         }));
-        if let Some(handle) = self.0.handle.upgrade().map(Connection) {
+        if let Some(handle) = connection {
             for interface in device.0.interfaces.values() {
                 handle
                     .0
@@ -677,6 +735,7 @@ pub enum EisRequest {
     TouchDown(TouchDown),
     TouchUp(TouchUp),
     TouchMotion(TouchMotion),
+    TouchCancel(TouchCancel),
 }
 
 impl EisRequest {
@@ -695,7 +754,33 @@ impl EisRequest {
             Self::TouchDown(evt) => Some(&mut evt.time),
             Self::TouchUp(evt) => Some(&mut evt.time),
             Self::TouchMotion(evt) => Some(&mut evt.time),
-            _ => None,
+            Self::TouchCancel(evt) => Some(&mut evt.time),
+            Self::Disconnect
+            | Self::Bind(_)
+            | Self::Frame(_)
+            | Self::DeviceStartEmulating(_)
+            | Self::DeviceStopEmulating(_) => None,
+        }
+    }
+
+    pub fn device(&self) -> Option<&Device> {
+        match self {
+            Self::Frame(evt) => Some(&evt.device),
+            Self::DeviceStartEmulating(evt) => Some(&evt.device),
+            Self::DeviceStopEmulating(evt) => Some(&evt.device),
+            Self::PointerMotion(evt) => Some(&evt.device),
+            Self::PointerMotionAbsolute(evt) => Some(&evt.device),
+            Self::Button(evt) => Some(&evt.device),
+            Self::ScrollDelta(evt) => Some(&evt.device),
+            Self::ScrollStop(evt) => Some(&evt.device),
+            Self::ScrollCancel(evt) => Some(&evt.device),
+            Self::ScrollDiscrete(evt) => Some(&evt.device),
+            Self::KeyboardKey(evt) => Some(&evt.device),
+            Self::TouchDown(evt) => Some(&evt.device),
+            Self::TouchUp(evt) => Some(&evt.device),
+            Self::TouchMotion(evt) => Some(&evt.device),
+            Self::TouchCancel(evt) => Some(&evt.device),
+            Self::Disconnect | Self::Bind(_) => None,
         }
     }
 }
@@ -815,6 +900,13 @@ pub struct TouchUp {
     pub touch_id: u32,
 }
 
+#[derive(Clone, Debug, PartialEq)]
+pub struct TouchCancel {
+    pub device: Device,
+    pub time: u64,
+    pub touch_id: u32,
+}
+
 pub trait SeatEvent {
     fn seat(&self) -> &Seat;
 }
